/*
 * Copyright (c) 2001 Fenris, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * File:  filter.c
 *
 * Purpose:  This source file contains the definition for filter.  It is a 
 * nested switch/case structure that depends the options maintained in a 
 * global filter object that is accessed by the gui interface.
 *
 */


#include <math.h>
#include <pcap.h>
#include <string.h>

#include "filter.h"
#include "globals.h"
#include "headers.h"


void filter(Packet_Headers* p, Net_Obj** obj)
{
    Filter_Opts* filter_opts;
    filter_opts = malloc(sizeof(Filter_Opts));

    if (p == NULL) {
        return;
    }

    /* Code starts to get ugly here if we put mutex_unlock calls as the first
     * statement inside each case... copy the filter_opts global to a local
     * malloced object and use that in the switch instead?
     */

    pthread_mutex_lock(filter_mutex);
    memcpy((void *) filter_opts, (void *) opts, sizeof(Filter_Opts)-1 );    
    pthread_mutex_unlock(filter_mutex);

    switch (filter_opts->menu) {
    case all :
        if (p->type == arp_t) {
            if (valid_arp_addr(p, filter_opts) == true) {
                create_arp_obj(p, obj);
            }
        } else if ((p->type == tcp_t) || (p->type == udp_t) ||
                   (p->type == icmp_t) || (p->type == igmp_t)) {
            if (valid_ip_addr(p, filter_opts) == true) {
                create_ip_obj(p, obj);
            }
        }
        break;
    case arp_m :
        switch (filter_opts->net_submenu.arp) {
        case arp_all:
            if (p->type == arp_t) {
                if (valid_arp_addr(p, filter_opts) == true) {
                    create_arp_obj(p, obj);
                }
            }
            break;
        default:
            break;
        }
        break;
    case ip_m : 
        switch (filter_opts->net_submenu.ip) {
        case ip_all :
            if ((p->type == tcp_t) || (p->type == udp_t) ||
                (p->type == icmp_t) || (p->type == igmp_t)) {
                if (valid_ip_addr(p, filter_opts) == true) {
                    create_ip_obj(p, obj);
                }
            }
            break;
    case icmp_m :     
            switch (filter_opts->net_submenu.ip_submenu.icmp) {
            case icmp_all :
                if (p->type == icmp_t) {
                    if (valid_ip_addr(p, filter_opts) == true) {
                        create_ip_obj(p, obj);
                    }
                }
                break;
            case icmp_rqst :
                if (p->type == icmp_t) {
                    if (p->tl.icmp->icmp_typ == ICMP_ECHO) {
                        if (valid_ip_addr(p, filter_opts) == true) {	
                            create_ip_obj(p, obj);
                        }
                    }
                }
                break;
            case icmp_rply :
                if (p->type == icmp_t) {
                    if (p->tl.icmp->icmp_typ == ICMP_ECHOREPLY) {
                        if (valid_ip_addr(p, filter_opts) == true) {
                            create_ip_obj(p, obj);
                        }
                    }
                }
                break;
            case icmp_pair :
                if (p->type == icmp_t) {
                    if ((p->tl.icmp->icmp_typ == ICMP_ECHO) ||
                        (p->tl.icmp->icmp_typ == ICMP_ECHO)) {
                        if (valid_ip_addr(p, filter_opts) == true) {
                            create_ip_obj(p, obj);
                        }
                    }
                }
                break;
            default :
                break;
            }
            break;
        case igmp_m :
            switch (filter_opts->net_submenu.ip_submenu.igmp) {
            case igmp_all :
                if (p->type == igmp_t) {
                    if (valid_ip_addr(p, filter_opts) == true) {
                        create_ip_obj(p, obj);
                    }
                }
                break;
            default :
                break;
            }
            break;
        case tcp_m :
            switch (filter_opts->net_submenu.ip_submenu.tcp) {
            case tcp_all :
                if (p->type == tcp_t) {
                    if (valid_ip_addr(p, filter_opts) == true) {
                        create_ip_obj(p, obj);
                    }
                }
                break;
            case tcp_http :
                if (p->type == tcp_t) {
                    if ((p->tl.tcp->tcp_src_port == 80) ||
                        (p->tl.tcp->tcp_dst_port == 80)) {
                        if (valid_ip_addr(p, filter_opts) == true) {
                            create_ip_obj(p, obj);
                        }
                    }
                }
                break;
            case tcp_ftp :
                if (p->type == tcp_t) {
                    if ((p->tl.tcp->tcp_src_port == 21) ||
                        (p->tl.tcp->tcp_dst_port == 21)) {
                        if (valid_ip_addr(p, filter_opts) == true) {
                            create_ip_obj(p, obj);
                        }
                    }
                }
                break;
            case tcp_ssh :
                if (p->type == tcp_t) {
                    if ((p->tl.tcp->tcp_src_port == 22) ||
                        (p->tl.tcp->tcp_dst_port == 22)) {
                        if (valid_ip_addr(p, filter_opts) == true) {
                            create_ip_obj(p, obj);
                        }
                    }
                }
                break;
            default :
                break;
            }
            break;
        case udp_m :
            switch (filter_opts->net_submenu.ip_submenu.udp) {
            case udp_all :
                if (p->type == udp_t) {
                    if (valid_ip_addr(p, filter_opts) == true) {
                        create_ip_obj(p, obj);
                    }
                }
                break;
            default :
                break;
            }  
            break;
        default :
            break;
        }
        break;
    default:
        break;
    }
    free(filter_opts);

    return;
}


enum bool valid_arp_addr (Packet_Headers* p, Filter_Opts* filter_opts)
{
    int			tmp_src_addr;
    int			tmp_dst_addr;

    tmp_src_addr = (p->nl.arp->arp_sdr_ip[0] << 24) |
        (p->nl.arp->arp_sdr_ip[1] << 16) |
        (p->nl.arp->arp_sdr_ip[2] <<  8) |
        (p->nl.arp->arp_sdr_ip[3]);
    tmp_dst_addr = (p->nl.arp->arp_tgt_ip[0] << 24) |
        (p->nl.arp->arp_tgt_ip[1] << 16) |
        (p->nl.arp->arp_tgt_ip[2] <<  8) |
        (p->nl.arp->arp_tgt_ip[3]);
    if ((filter_opts->net_submenu.filter_on_src_addr == false) &&
        (filter_opts->net_submenu.filter_on_dst_addr == false)) {
        return true;
    } else if (((filter_opts->net_submenu.filter_on_src_addr == TRUE) &&
                (filter_opts->net_submenu.src_addr == tmp_src_addr)) ||
               ((filter_opts->net_submenu.filter_on_dst_addr == TRUE) &&
                (filter_opts->net_submenu.dst_addr == tmp_dst_addr))) {
        return true;
    } else {
        return false;
    }
}


enum bool valid_ip_addr (Packet_Headers* p, Filter_Opts* filter_opts)
{
    unsigned int mask = 0xFFFFFFFF; 
    unsigned int src_mask = mask ^ ((unsigned int)pow(2, (32 - filter_opts->net_submenu.src_cidr)) - 1);
    unsigned int dst_mask = mask ^ ((unsigned int)pow(2, (32 - filter_opts->net_submenu.dst_cidr)) - 1);
    /*
     printf("mask = %X\n", mask);
     printf("src_cidr = %d\n", filter_opts->net_submenu.src_cidr);
     printf("src_mask = %X\n", src_mask);
     printf("src_addr = %X\n", p->nl.ip->ip_src);
     */
   
    if (valid_port(p, filter_opts) == false) return false;

    if ((filter_opts->net_submenu.filter_on_src_addr == false) &&
        (filter_opts->net_submenu.filter_on_dst_addr == false)) {
        return true;
    } else if (((filter_opts->net_submenu.filter_on_src_addr == TRUE) &&
                ((filter_opts->net_submenu.src_addr & src_mask) == 
                 (p->nl.ip->ip_src & src_mask))) || 
               ((filter_opts->net_submenu.filter_on_dst_addr == TRUE) &&
                ((filter_opts->net_submenu.dst_addr & dst_mask) == 
                 (p->nl.ip->ip_dst & dst_mask)))) {
        return true;
    } else {
        return false;
    }
}

enum bool valid_port (Packet_Headers* p, Filter_Opts* filter_opts)
{
    enum bool           valid_src, valid_dst;

    if (filter_opts->net_submenu.filter_on_src_ports == true) {
        if ((p->type == tcp_t) || (p->type == udp_t)) {  
            if (p->tl.tcp != NULL) {
                if (filter_opts->net_submenu.
                    src_ports[p->tl.tcp->tcp_src_port] == true) {
                    valid_src = true;
                }
            } else if (p->tl.udp != NULL) {
                if (filter_opts->net_submenu.
                    src_ports[p->tl.udp->udp_src_port] == true) {
                    valid_src = true;
                }
            }
        } else if ((p->type == icmp_t) || (p->type == igmp_t)) {
            valid_src = false;
        }
    } else {
        valid_src = true;
    }

    if (filter_opts->net_submenu.filter_on_dst_ports == true) {
        if ((p->type == tcp_t) || (p->type == udp_t)) {
            if (p->tl.tcp != NULL) {
                if (filter_opts->net_submenu.
                    dst_ports[p->tl.tcp->tcp_dst_port] == true) {
                    valid_dst = true;
                }
            } else if (p->tl.udp != NULL) {
                if (filter_opts->net_submenu.
                    dst_ports[p->tl.udp->udp_dst_port] == true) {
                    valid_dst = true;
                }
            }
        } else if ((p->type == icmp_t) || (p->type == igmp_t)) {
            valid_dst = false;
        }
    } else {
        valid_dst = true;
    }

    if ((valid_src == true) && (valid_dst == true)) {
        return true;
    }

    return false;
}

void create_ip_obj (Packet_Headers* p, Net_Obj** obj)
{
    if ((*obj = malloc(sizeof(Net_Obj))) == NULL)
        die("malloc");

    (*obj)->src = p->nl.ip->ip_src;
    (*obj)->dst = p->nl.ip->ip_dst;

    return;
}


void create_arp_obj (Packet_Headers* p, Net_Obj** obj)
{
    if ((*obj = malloc(sizeof(Net_Obj))) == NULL)
        die("malloc");

    (*obj)->src = (p->nl.arp->arp_sdr_ip[0] << 24) |
        (p->nl.arp->arp_sdr_ip[1] << 16) |
        (p->nl.arp->arp_sdr_ip[2] <<  8) |
        (p->nl.arp->arp_sdr_ip[3]);
    (*obj)->dst = (p->nl.arp->arp_tgt_ip[0] << 24) |
        (p->nl.arp->arp_tgt_ip[1] << 16) |
        (p->nl.arp->arp_tgt_ip[2] <<  8) |
        (p->nl.arp->arp_tgt_ip[3]);
    return;
}

